@@ -1,3 +1,34 @@
+/*
+ ***** BEGIN LICENSE BLOCK *****
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
+ *
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ * 
+ * Copyright (C) 2008-2011 Laurent Gautier
+ *
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
+ *
+ * ***** END LICENSE BLOCK ***** */
+
+
 /* --- NA values --- */
 
 #include <longintrepr.h>
@@ -11,7 +42,11 @@ NAInteger_repr(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)  
     repr = PyString_FromString("NA_integer_");
+#else
+    repr = PyUnicode_FromString("NA_integer_");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -22,7 +57,11 @@ NA_str(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)  
     repr = PyString_FromString("NA");
+#else
+    repr = PyUnicode_FromString("NA");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -53,14 +92,17 @@ NA_ternaryfunc(PyObject *self, PyObject *obj1, PyObject *obj2)
 static int
 NA_nonzero(PyObject *self)
 {
-  return 1;
+  PyErr_Format(PyExc_ValueError, "NA values cannot be evaluated as booleans.");
+  return 0;
 }
 
 static PyNumberMethods NAInteger_NumberMethods = {
   (binaryfunc)NA_binaryfunc, /* nb_add */
   (binaryfunc)NA_binaryfunc, /* nb_subtract; */
   (binaryfunc)NA_binaryfunc, /* nb_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
   (binaryfunc)NA_binaryfunc, /* nb_divide; */
+#endif
   (binaryfunc)NA_binaryfunc, /* nb_remainder; */
   (binaryfunc)NA_binaryfunc, /* nb_divmod; */
   (ternaryfunc)NA_ternaryfunc, /* nb_power; */
@@ -74,19 +116,29 @@ static PyNumberMethods NAInteger_NumberMethods = {
   (binaryfunc) NA_binaryfunc, /*  nb_and; */
   (binaryfunc) NA_binaryfunc, /*  nb_xor; */
   (binaryfunc) NA_binaryfunc, /* nb_or; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, //(coerce) NA_coerce, /* coercion nb_coerce;       -- Used by the coerce() function */
+#endif
   (unaryfunc) NA_unaryfunc, /* nb_int; */
+#if (PY_VERSION_HEX < 0x03010000)
   (unaryfunc) NA_unaryfunc, /* nb_long; */
+#else
+  NULL, /* reserved */
+#endif
   (unaryfunc) NA_unaryfunc, /* nb_float; */
+#if (PY_VERSION_HEX < 0x03010000)
   (unaryfunc) NA_unaryfunc, /* nb_oct; */
   (unaryfunc) NA_unaryfunc, /* nb_hex; */
+#endif
   /* Added in release 2.0 */
-  0, /* nb_inplace_add; */
-  0, /* nb_inplace_subtract; */
-  0, /* nb_inplace_multiply; */
-  0, /* nb_inplace_divide; */
-  0, /* nb_inplace_remainder; */
-  0, /* nb_inplace_power; */
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_add; */
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_subtract; */
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_divide; */
+#endif
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_remainder; */
+  (ternaryfunc)NA_ternaryfunc, /* nb_inplace_power; */
   0, /* nb_inplace_lshift; */
   0, /* nb_inplace_rshift; */
   0, /* nb_inplace_and; */
@@ -95,8 +147,8 @@ static PyNumberMethods NAInteger_NumberMethods = {
   /* Added in release 2.2 */
   (binaryfunc) NA_binaryfunc, /* nb_floor_divide; */
   (binaryfunc) NA_binaryfunc, /* nb_true_divide; */
-  0, /* nb_inplace_floor_divide; */
-  0, /* nb_inplace_true_divide; */
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_floor_divide; */
+  (binaryfunc)NA_binaryfunc, /* nb_inplace_true_divide; */
   /* Added in release 2.5 */
 #if PY_VERSION_HEX >= 0x02050000
   (unaryfunc) NA_unaryfunc /* nb_index; */
@@ -111,10 +163,14 @@ NAInteger_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 static PyTypeObject NAInteger_Type = {
         /* The ob_type field must be initialized in the module init function
          * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
         PyObject_HEAD_INIT(NULL)
         0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
         "rpy2.rinterface.NAIntegerType",       /*tp_name*/
-        sizeof(PyObject),   /*tp_basicsize*/
+        sizeof(PyLongObject),   /*tp_basicsize*/
         0,                      /*tp_itemsize*/
         /* methods */
         0, /*tp_dealloc*/
@@ -123,7 +179,7 @@ static PyTypeObject NAInteger_Type = {
         0,                      /*tp_setattr*/
         0,                      /*tp_compare*/
         NAInteger_repr,                      /*tp_repr*/
-        &NAInteger_NumberMethods,                      /*tp_as_number*/
+        &NAInteger_NumberMethods,            /*tp_as_number*/
         0,                      /*tp_as_sequence*/
         0,                      /*tp_as_mapping*/
         0,                      /*tp_hash*/
@@ -132,7 +188,11 @@ static PyTypeObject NAInteger_Type = {
         0,                      /*tp_getattro*/
         0,                      /*tp_setattro*/
         0,                      /*tp_as_buffer*/
+#if (PY_VERSION_HEX < 0x03010000)
         Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES, /*tp_flags*/
+#else
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
+#endif
         NAInteger_Type_doc,                      /*tp_doc*/
         0,                      /*tp_traverse*/
         0,                      /*tp_clear*/
@@ -225,7 +285,11 @@ NALogical_repr(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)  
     repr = PyString_FromString("NA");
+#else
+    repr = PyUnicode_FromString("NA");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -235,7 +299,9 @@ static PyNumberMethods NALogical_NumberMethods = {
   0, /* nb_add */
   0, /* nb_subtract; */
   0, /* nb_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, /* nb_divide; */
+#endif
   0, /* nb_remainder; */
   0, /* nb_divmod; */
   0, /* nb_power; */
@@ -249,17 +315,27 @@ static PyNumberMethods NALogical_NumberMethods = {
   (binaryfunc) NA_binaryfunc, /*  nb_and; */
   (binaryfunc) NA_binaryfunc, /*  nb_xor; */
   (binaryfunc) NA_binaryfunc, /* nb_or; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, //(coerce) NA_coerce, /* coercion nb_coerce;       -- Used by the coerce() function */
+#endif
   0, /* nb_int; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, /* nb_long; */
+#else
+  NULL, /* reserved */
+#endif
   0, /* nb_float; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, /* nb_oct; */
   0, /* nb_hex; */
+#endif
   /* Added in release 2.0 */
   0, /* nb_inplace_add; */
   0, /* nb_inplace_subtract; */
   0, /* nb_inplace_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, /* nb_inplace_divide; */
+#endif
   0, /* nb_inplace_remainder; */
   0, /* nb_inplace_power; */
   0, /* nb_inplace_lshift; */
@@ -281,10 +357,14 @@ static PyNumberMethods NALogical_NumberMethods = {
 static PyTypeObject NALogical_Type = {
         /* The ob_type field must be initialized in the module init function
          * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
         PyObject_HEAD_INIT(NULL)
         0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
         "rpy2.rinterface.NALogicalType",       /*tp_name*/
-        sizeof(PyObject),   /*tp_basicsize*/
+        sizeof(PyLongObject),   /*tp_basicsize*/
         0,                      /*tp_itemsize*/
         /* methods */
         0, /*tp_dealloc*/
@@ -302,7 +382,11 @@ static PyTypeObject NALogical_Type = {
         0,                      /*tp_getattro*/
         0,                      /*tp_setattro*/
         0,                      /*tp_as_buffer*/
+#if (PY_VERSION_HEX < 0x03010000)
         Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES, /*tp_flags*/
+#else
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
+#endif
         NALogical_Type_doc,                      /*tp_doc*/
         0,                      /*tp_traverse*/
         0,                      /*tp_clear*/
@@ -334,8 +418,40 @@ PyDoc_STRVAR(NAReal_Type_doc,
 static PyObject*
 NAReal_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
-  RPY_NA_TP_NEW("NARealType", PyFloat_Type, PyFloat_FromDouble, 
-                (double)NA_REAL)
+  //printf("--->0x%llx\n", *(unsigned long long *)&(NAREAL_IEEE.value));
+  static PyObject *self = NULL;
+  static char *kwlist[] = {0};
+  PyObject *py_value;
+
+  assert(PyType_IsSubtype(type, &PyFloat_Type));
+
+  if (! PyArg_ParseTupleAndKeywords(args, kwds, "", kwlist)) {
+    return NULL;
+  }
+
+  if (self == NULL) {
+    py_value = PyFloat_FromDouble((double)(NAREAL_IEEE.value));
+    //(double)0x7ff00000000007a2
+    //NA_REAL
+    //py_value = PyFloat_FromDouble(NA_REAL);
+    if (py_value == NULL) {
+      return NULL;
+    }
+    assert(PyFloat_CheckExact(py_value));
+    self = type->tp_alloc(type, 0);
+    if (self == NULL) {
+      //printf("--->\n");
+      Py_DECREF(py_value);
+      return NULL;
+    }
+    ((PyFloatObject *)self)->ob_fval = ((PyFloatObject *)py_value)->ob_fval;
+    //((PyFloatObject *)self)->ob_fval = (double)(NAREAL_IEEE.value);
+    Py_DECREF(py_value);
+  }
+  Py_INCREF(self);
+  return self;
+  /* RPY_NA_TP_NEW("NARealType", PyFloat_Type, PyFloat_FromDouble, */
+  /* 		  NA_REAL); */
 }
 
 static PyObject*
@@ -349,7 +465,11 @@ NAReal_repr(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)  
     repr = PyString_FromString("NA_real_");
+#else
+    repr = PyUnicode_FromString("NA_real_");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -360,7 +480,9 @@ static PyNumberMethods NAReal_NumberMethods = {
   (binaryfunc)NA_binaryfunc, /* nb_add */
   (binaryfunc)NA_binaryfunc, /* nb_subtract; */
   (binaryfunc)NA_binaryfunc, /* nb_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
   (binaryfunc)NA_binaryfunc, /* nb_divide; */
+#endif
   (binaryfunc)NA_binaryfunc, /* nb_remainder; */
   (binaryfunc)NA_binaryfunc, /* nb_divmod; */
   (ternaryfunc)NA_ternaryfunc, /* nb_power; */
@@ -374,17 +496,27 @@ static PyNumberMethods NAReal_NumberMethods = {
   0, /*  nb_and; */
   0, /*  nb_xor; */
   0, /* nb_or; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, //(coerce) NA_coerce, /* coercion nb_coerce;       -- Used by the coerce() function */
+#endif
   (unaryfunc) NA_unaryfunc, /* nb_int; */
+#if (PY_VERSION_HEX < 0x03010000)
   (unaryfunc) NA_unaryfunc, /* nb_long; */
+#else
+  NULL, /* reserved */
+#endif
   (unaryfunc) NA_unaryfunc, /* nb_float; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, /* nb_oct; */
   0, /* nb_hex; */
+#endif
   /* Added in release 2.0 */
   0, /* nb_inplace_add; */
   0, /* nb_inplace_subtract; */
   0, /* nb_inplace_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
   0, /* nb_inplace_divide; */
+#endif
   0, /* nb_inplace_remainder; */
   0, /* nb_inplace_power; */
   0, /* nb_inplace_lshift; */
@@ -407,10 +539,14 @@ static PyNumberMethods NAReal_NumberMethods = {
 static PyTypeObject NAReal_Type = {
         /* The ob_type field must be initialized in the module init function
          * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
         PyObject_HEAD_INIT(NULL)
         0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
         "rpy2.rinterface.NARealType",       /*tp_name*/
-        sizeof(PyObject),   /*tp_basicsize*/
+        sizeof(PyFloatObject),   /*tp_basicsize*/
         0,                      /*tp_itemsize*/
         /* methods */
         0, /*tp_dealloc*/
@@ -428,7 +564,11 @@ static PyTypeObject NAReal_Type = {
         0,                      /*tp_getattro*/
         0,                      /*tp_setattro*/
         0,                      /*tp_as_buffer*/
+#if (PY_VERSION_HEX < 0x03010000)
         Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES, /*tp_flags*/
+#else
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
+#endif
         NAReal_Type_doc,                      /*tp_doc*/
         0,                      /*tp_traverse*/
         0,                      /*tp_clear*/
@@ -451,7 +591,7 @@ static PyTypeObject NAReal_Type = {
         0                      /*tp_is_gc*/
 };
 
-/* NA Float / Real */
+/* NA Character */
 
 PyDoc_STRVAR(NACharacter_Type_doc,
 "Missing value for a string."
@@ -460,7 +600,11 @@ PyDoc_STRVAR(NACharacter_Type_doc,
 static PyObject*
 NACharacter_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
+#if (PY_VERSION_HEX < 0x03010000)
   RPY_NA_TP_NEW("NACharacterType", PyString_Type, PyString_FromString, "")
+#else
+    RPY_NA_TP_NEW("NACharacterType", PyUnicode_Type, PyUnicode_FromString, "")
+#endif
 }
 
 static PyObject*
@@ -475,7 +619,11 @@ NACharacter_repr(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)
     repr = PyString_FromString("NA_character_");
+#else
+    repr = PyUnicode_FromString("NA_character_");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -485,10 +633,18 @@ NACharacter_repr(PyObject *self)
 static PyTypeObject NACharacter_Type = {
         /* The ob_type field must be initialized in the module init function
          * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
         PyObject_HEAD_INIT(NULL)
         0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
         "rpy2.rinterface.NACharacterType",       /*tp_name*/
-        sizeof(PyObject),   /*tp_basicsize*/
+#if (PY_VERSION_HEX < 0x03010000)
+        sizeof(PyStringObject),             /*tp_basicsize*/
+#else
+	sizeof(PyUnicodeObject),             /*tp_basicsize*/
+#endif
         0,                      /*tp_itemsize*/
         /* methods */
         0, /*tp_dealloc*/
@@ -506,7 +662,11 @@ static PyTypeObject NACharacter_Type = {
         0,                      /*tp_getattro*/
         0,                      /*tp_setattro*/
         0,                      /*tp_as_buffer*/
+#if (PY_VERSION_HEX < 0x03010000)
         Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES, /*tp_flags*/
+#else
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
+#endif
         NACharacter_Type_doc,                      /*tp_doc*/
         0,                      /*tp_traverse*/
         0,                      /*tp_clear*/
@@ -517,7 +677,11 @@ static PyTypeObject NACharacter_Type = {
         0, //NAInteger_methods,           /*tp_methods*/
         0,                      /*tp_members*/
         0,                      /*tp_getset*/
+#if (PY_VERSION_HEX < 0x03010000)
         &PyString_Type,             /*tp_base*/
+#else
+	&PyUnicode_Type,
+#endif
         0,                      /*tp_dict*/
         0,                      /*tp_descr_get*/
         0,                      /*tp_descr_set*/
@@ -530,6 +694,166 @@ static PyTypeObject NACharacter_Type = {
 };
 
 
+/* NA Complex */
+
+PyDoc_STRVAR(NAComplex_Type_doc,
+"Missing value for a complex in R."
+);
+
+static PyObject*
+NAComplex_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+  //static PyObject *self = NULL;
+  //static char *kwlist[] = {0};
+
+  Py_complex pyvalue = {(double)NAREAL_IEEE.value,
+			(double)NAREAL_IEEE.value};
+
+   
+  assert(PyType_IsSubtype(type, &PyComplex_Type));
+
+  RPY_NA_TP_NEW('Complex', PyComplex_Type, 
+		PyComplex_FromCComplex, pyvalue);
+
+}
+
+static PyObject*
+NAComplex_New(int new)
+{
+  RPY_NA_NEW(NAComplex_Type, NAComplex_tp_new)
+}
+
+static PyObject*
+NAComplex_repr(PyObject *self)
+{
+  static PyObject* repr = NULL;
+  if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)
+    repr = PyString_FromString("NA_complex_");
+#else
+    repr = PyUnicode_FromString("NA_complex_");
+#endif
+  }
+  Py_XINCREF(repr);
+  return repr;
+}
+
+
+static PyNumberMethods NAComplex_NumberMethods = {
+  (binaryfunc)NA_binaryfunc, /* nb_add */
+  (binaryfunc)NA_binaryfunc, /* nb_subtract; */
+  (binaryfunc)NA_binaryfunc, /* nb_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
+  (binaryfunc)NA_binaryfunc, /* nb_divide; */
+#endif
+  (binaryfunc)NA_binaryfunc, /* nb_remainder; */
+  (binaryfunc)NA_binaryfunc, /* nb_divmod; */
+  (ternaryfunc)NA_ternaryfunc, /* nb_power; */
+  (unaryfunc) NA_unaryfunc, /* nb_negative; */
+  (unaryfunc) NA_unaryfunc, /* nb_positive; */
+  (unaryfunc) NA_unaryfunc, /* nb_absolute; */
+  (inquiry) NA_nonzero, /* nb_nonzero;   -- Used by PyObject_IsTrue */
+  0, /* nb_invert; */
+  0, /* nb_lshift; */
+  0, /* nb_rshift; */
+  0, /*  nb_and; */
+  0, /*  nb_xor; */
+  0, /* nb_or; */
+#if (PY_VERSION_HEX < 0x03010000)
+  0, //(coerce) NA_coerce, /* coercion nb_coerce;       -- Used by the coerce() function */
+#endif
+  (unaryfunc) NA_unaryfunc, /* nb_int; */
+#if (PY_VERSION_HEX < 0x03010000)
+  (unaryfunc) NA_unaryfunc, /* nb_long; */
+#else
+  NULL, /* reserved */
+#endif
+  (unaryfunc) NA_unaryfunc, /* nb_float; */
+#if (PY_VERSION_HEX < 0x03010000)
+  0, /* nb_oct; */
+  0, /* nb_hex; */
+#endif
+  /* Added in release 2.0 */
+  0, /* nb_inplace_add; */
+  0, /* nb_inplace_subtract; */
+  0, /* nb_inplace_multiply; */
+#if (PY_VERSION_HEX < 0x03010000)
+  0, /* nb_inplace_divide; */
+#endif
+  0, /* nb_inplace_remainder; */
+  0, /* nb_inplace_power; */
+  0, /* nb_inplace_lshift; */
+  0, /* nb_inplace_rshift; */
+  0, /* nb_inplace_and; */
+  0, /* nb_inplace_xor; */
+  0, /* nb_inplace_or; */
+  /* Added in release 2.2 */
+  (binaryfunc) NA_binaryfunc, /* nb_floor_divide; */
+  (binaryfunc) NA_binaryfunc, /* nb_true_divide; */
+  0, /* nb_inplace_floor_divide; */
+  0, /* nb_inplace_true_divide; */
+  /* Added in release 2.5 */
+#if PY_VERSION_HEX >= 0x02050000
+  0 /* nb_index; */
+#endif
+};
+
+
+static PyTypeObject NAComplex_Type = {
+        /* The ob_type field must be initialized in the module init function
+         * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
+        PyObject_HEAD_INIT(NULL)
+        0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
+        "rpy2.rinterface.NAComplexType",       /*tp_name*/
+        sizeof(PyComplexObject),   /*tp_basicsize*/
+        0,                      /*tp_itemsize*/
+        /* methods */
+        0, /*tp_dealloc*/
+        0,                      /*tp_print*/
+        0,                      /*tp_getattr*/
+        0,                      /*tp_setattr*/
+        0,                      /*tp_compare*/
+        NAComplex_repr,                      /*tp_repr*/
+        &NAComplex_NumberMethods,                      /*tp_as_number*/
+        0,                      /*tp_as_sequence*/
+        0,                      /*tp_as_mapping*/
+        0,                      /*tp_hash*/
+        0,                      /*tp_call*/
+        NA_str,                      /*tp_str*/
+        0,                      /*tp_getattro*/
+        0,                      /*tp_setattro*/
+        0,                      /*tp_as_buffer*/
+#if (PY_VERSION_HEX < 0x03010000)
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES, /*tp_flags*/
+#else
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
+#endif
+        NAComplex_Type_doc,                      /*tp_doc*/
+        0,                      /*tp_traverse*/
+        0,                      /*tp_clear*/
+        0,                      /*tp_richcompare*/
+        0,                      /*tp_weaklistoffset*/
+        0,                      /*tp_iter*/
+        0,                      /*tp_iternext*/
+        0, //NAInteger_methods,           /*tp_methods*/
+        0,                      /*tp_members*/
+        0,                      /*tp_getset*/
+        &PyComplex_Type,             /*tp_base*/
+        0,                      /*tp_dict*/
+        0,                      /*tp_descr_get*/
+        0,                      /*tp_descr_set*/
+        0,                      /*tp_dictoffset*/
+        0, //(initproc)ClosureSexp_init,                      /*tp_init*/
+        0,                      /*tp_alloc*/
+        NAComplex_tp_new,                      /*tp_new*/
+        0,                      /*tp_free*/
+        0                      /*tp_is_gc*/
+};
+
 
 /* Missing parameter value (not an NA in the usual sense) */
 
@@ -537,7 +861,11 @@ PyDoc_STRVAR(MissingArg_Type_doc,
 "Missing argument (in a function call)."
 );
 
+#if (PY_VERSION_HEX < 0x03010000)
 staticforward PyTypeObject MissingArg_Type;
+#else
+static PyTypeObject MissingArg_Type;
+#endif
 
 static PyObject*
 MissingArgType_tp_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
@@ -574,7 +902,11 @@ MissingArgType_repr(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)
     repr = PyString_FromString("rpy2.rinterface.MissingArg");
+#else
+    repr = PyUnicode_FromString("rpy2.rinterface.MissingArg");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -585,7 +917,11 @@ MissingArgType_str(PyObject *self)
 {
   static PyObject* repr = NULL;
   if (repr == NULL) {
+#if (PY_VERSION_HEX < 0x03010000)  
     repr = PyString_FromString("MissingArg");
+#else
+    repr = PyUnicode_FromString("MissingArg");
+#endif
   }
   Py_XINCREF(repr);
   return repr;
@@ -594,8 +930,12 @@ MissingArgType_str(PyObject *self)
 static PyTypeObject MissingArg_Type = {
         /* The ob_type field must be initialized in the module init function
          * to be portable to Windows without using C++. */
+#if (PY_VERSION_HEX < 0x03010000)
         PyObject_HEAD_INIT(NULL)
         0,                      /*ob_size*/
+#else
+	PyVarObject_HEAD_INIT(NULL, 0)
+#endif
         "rpy2.rinterface.MissingArgType",       /*tp_name*/
         sizeof(PySexpObject),   /*tp_basicsize*/
         0,                      /*tp_itemsize*/
@@ -615,7 +955,11 @@ static PyTypeObject MissingArg_Type = {
         0,                      /*tp_getattro*/
         0,                      /*tp_setattro*/
         0,                      /*tp_as_buffer*/
+#if (PY_VERSION_HEX < 0x03010000)
         Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES, /*tp_flags*/
+#else
+        Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /*tp_flags*/
+#endif
         MissingArg_Type_doc,                      /*tp_doc*/
         0,                      /*tp_traverse*/
         0,                      /*tp_clear*/
