@@ -1,9 +1,12 @@
 import rpy2.robjects.methods
 import rpy2.robjects as robjects
 import rpy2.robjects.conversion as conversion
+import rpy2.rinterface as rinterface
 from rpy2.robjects.packages import importr
 import copy
 
+NULL = robjects.NULL
+
 #getmethod = robjects.baseenv.get("getMethod")
 
 rimport = robjects.baseenv.get('library')
@@ -14,7 +17,7 @@ ggplot2_env = robjects.baseenv['as.environment']('package:ggplot2')
 StrVector = robjects.StrVector
 
 def as_symbol(x):
-   res = robjects.baseenv["parse"](text = x)
+   res = rinterface.parse(x)
    return res
 
 class GGPlot(robjects.RObject):
@@ -28,7 +31,7 @@ class GGPlot(robjects.RObject):
         res = cls(cls._constructor(data))
         return res
     
-    def plot(self, vp = robjects.NULL):
+    def plot(self, vp = robjects.constants.NULL):
         self._rprint(self, vp = vp)
 
     def __add__(self, obj):
@@ -409,6 +412,7 @@ class YLim(Scale):
    _constructor = ggplot2_env['ylim']
 ylim = YLim.new
 
+
 class ScaleXContinuous(ScaleX):
    _constructor = ggplot2_env['scale_x_continuous']
 scale_x_continuous = ScaleXContinuous.new
@@ -572,13 +576,17 @@ class ScaleShapeManual(ScaleShape):
    _constructor = ggplot2_env['scale_shape_manual']
 scale_shape_manual = ScaleShapeManual.new
 
+
 class Options(robjects.Vector):
-    pass
+   def __init__(self, obj):
+      self.__sexp__ = obj.__sexp__
+
+   def __repr__(self):
+      s = '<instance of %s : %i>' %(type(self), id(self)) 
+      return s
+
 class Theme(Options):
-    @classmethod
-    def new(cls, base_size = 12):
-        res = cls(cls._constructor(base_size = base_size))
-        return res
+   pass
 
 class ThemeBlank(Theme):
     _constructor = ggplot2.theme_blank
@@ -586,19 +594,24 @@ class ThemeBlank(Theme):
     def new(cls):
         res = cls(cls._constructor())
         return res
+
 theme_blank = ThemeBlank.new
 
 theme_get = ggplot2.theme_get
 
 class ThemeGrey(Theme):
     _constructor = ggplot2.theme_grey
+    @classmethod
+    def new(cls, base_size = 12):
+       res = cls(cls._constructor(base_size = base_size))
+       return res
 
 theme_grey = ThemeGrey.new
 
 class ThemeRect(Theme):
     _constructor = ggplot2.theme_rect
     @classmethod
-    def new(cls, fill = robjects.NA_bool, colour = "black", 
+    def new(cls, fill = robjects.NA_Logical, colour = "black", 
             size = 0.5, linetype = 1):
        res = cls(cls._constructor(fill = fill, colour = colour, 
                                   size = size, linetype = linetype))
@@ -607,7 +620,11 @@ theme_rect = ThemeRect.new
 
 class ThemeSegment(Theme):
     _constructor = ggplot2.theme_rect
-    
+    @classmethod
+    def new(cls, colour = 'black', size = 0.5, linetype = 1):
+       res = cls(cls._constructor(colour = colour, size = size,
+                                  linetype = linetype))
+       return res
 theme_segment = ThemeSegment.new
 
 # Theme text is not a vector :/
@@ -625,14 +642,28 @@ theme_text = ThemeText.new
 
 class ThemeBW(Theme):
     _constructor = ggplot2.theme_bw
+    @classmethod
+    def new(cls, base_size = 12):
+       res = cls(cls._constructor(base_size = base_size))
+       return res
+
 theme_bw = ThemeBW.new
 
 class ThemeGray(Theme):
     _constructor = ggplot2.theme_gray
+    @classmethod
+    def new(cls, base_size = 12):
+       res = cls(cls._constructor(base_size = base_size))
+       return res
 theme_gray = ThemeGray.new
 
 class ThemeLine(Theme):
     _constructor = ggplot2.theme_line
+    @classmethod
+    def new(cls, colour = 'black', size = 0.5, linetype = 1):
+       res = cls(cls._constructor(colour = colour, size = size,
+                                  linetype = linetype))
+       return res
 theme_line = ThemeLine.new
 
 #theme_render
@@ -652,7 +683,11 @@ def ggplot2_conversion(robj):
 
     pyobj = original_conversion(robj)
 
-    if 'ggplot' in pyobj.rclass:
+    rcls = pyobj.rclass
+    if rcls is NULL:
+       rcls = (None, )
+
+    if 'ggplot' in rcls:
        pyobj = GGPlot(pyobj)
 
     return pyobj
