@@ -1,9 +1,12 @@
 import unittest
 import rpy2.robjects as robjects
 ri = robjects.rinterface
-import array
+import array, time, sys
 import rpy2.rlike.container as rlc
 
+IS_PYTHON3 = sys.version_info[0] == 3
+
+
 rlist = robjects.baseenv["list"]
 
 class VectorTestCase(unittest.TestCase):
@@ -12,7 +15,7 @@ class VectorTestCase(unittest.TestCase):
         identical = ri.baseenv["identical"]
         py_a = array.array('i', [1,2,3])
         ro_v = robjects.Vector(py_a)
-        self.assertEquals(ro_v.typeof, ri.INTSXP)
+        self.assertEqual(ro_v.typeof, ri.INTSXP)
         
         ri_v = ri.SexpVector(py_a, ri.INTSXP)
         ro_v = robjects.Vector(ri_v)
@@ -20,49 +23,38 @@ class VectorTestCase(unittest.TestCase):
         self.assertTrue(identical(ro_v, ri_v)[0])
 
         del(ri_v)
-        self.assertEquals(ri.INTSXP, ro_v.typeof)
+        self.assertEqual(ri.INTSXP, ro_v.typeof)
 
     def testNewStrVector(self):
         vec = robjects.StrVector(['abc', 'def'])
-        self.assertEquals('abc', vec[0])
-        self.assertEquals('def', vec[1])
-        self.assertEquals(2, len(vec))
+        self.assertEqual('abc', vec[0])
+        self.assertEqual('def', vec[1])
+        self.assertEqual(2, len(vec))
 
     def testNewIntVector(self):
         vec = robjects.IntVector([123, 456])
-        self.assertEquals(123, vec[0])
-        self.assertEquals(456, vec[1])
-        self.assertEquals(2, len(vec))
+        self.assertEqual(123, vec[0])
+        self.assertEqual(456, vec[1])
+        self.assertEqual(2, len(vec))
 
     def testNewFloatVector(self):
         vec = robjects.FloatVector([123.0, 456.0])
-        self.assertEquals(123.0, vec[0])
-        self.assertEquals(456.0, vec[1])
-        self.assertEquals(2, len(vec))
+        self.assertEqual(123.0, vec[0])
+        self.assertEqual(456.0, vec[1])
+        self.assertEqual(2, len(vec))
 
     def testNewBoolVector(self):
         vec = robjects.BoolVector([True, False])
-        self.assertEquals(True, vec[0])
-        self.assertEquals(False, vec[1])
-        self.assertEquals(2, len(vec))
-
-    def testNewFactorVector(self):
-        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
-        self.assertEquals(6, len(vec))
-
-    def testFactorVector_isordered(self):
-        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
-        self.assertFalse(vec.isordered)
-
-    def testFactorVector_nlevels(self):
-        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
-        self.assertEquals(3, vec.nlevels)
-
-    def testFactorVector_levels(self):
-        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
-        self.assertEquals(3, len(vec.levels))
-        self.assertEquals(set(('a','b','c')), set(tuple(vec.levels)))
+        self.assertEqual(True, vec[0])
+        self.assertEqual(False, vec[1])
+        self.assertEqual(2, len(vec))
 
+    def testNewListVector(self):
+        vec = robjects.ListVector({'a': 1, 'b': 2})
+        self.assertTrue('a' in vec.names)
+        self.assertTrue('b' in vec.names)
+        self.assertEqual(2, len(vec))
+        self.assertEqual(2, len(vec.names))
 
     def testAddOperators(self):
         seq_R = robjects.r["seq"]
@@ -70,39 +62,39 @@ class VectorTestCase(unittest.TestCase):
         mySeqB = seq_R(5, 7)
         mySeqAdd = mySeqA + mySeqB
 
-        self.assertEquals(len(mySeqA)+len(mySeqB), len(mySeqAdd))
+        self.assertEqual(len(mySeqA)+len(mySeqB), len(mySeqAdd))
 
         for i, li in enumerate(mySeqA):
-            self.assertEquals(li, mySeqAdd[i])       
+            self.assertEqual(li, mySeqAdd[i])       
         for j, li in enumerate(mySeqB):
-            self.assertEquals(li, mySeqAdd[i+j+1])
+            self.assertEqual(li, mySeqAdd[i+j+1])
 
     def testRAddOperators(self):
         seq_R = robjects.r["seq"]
         mySeq = seq_R(0, 10)
         mySeqAdd = mySeq.ro + 2
         for i, li in enumerate(mySeq):
-            self.assertEquals(li + 2, mySeqAdd[i])
+            self.assertEqual(li + 2, mySeqAdd[i])
 
     def testRMultOperators(self):
         seq_R = robjects.r["seq"]
         mySeq = seq_R(0, 10)
         mySeqAdd = mySeq.ro + mySeq
         for i, li in enumerate(mySeq):
-            self.assertEquals(li * 2, mySeqAdd[i])
+            self.assertEqual(li * 2, mySeqAdd[i])
 
     def testRPowerOperator(self):
         seq_R = robjects.r["seq"]
         mySeq = seq_R(0, 10)
         mySeqPow = mySeq.ro ** 2
         for i, li in enumerate(mySeq):
-            self.assertEquals(li ** 2, mySeqPow[i])
+            self.assertEqual(li ** 2, mySeqPow[i])
 
  
     def testGetItem(self):
         letters = robjects.baseenv["letters"]
-        self.assertEquals('a', letters[0])
-        self.assertEquals('z', letters[25])
+        self.assertEqual('a', letters[0])
+        self.assertEqual('z', letters[25])
 
     def testGetItemOutOfBounds(self):
         letters = robjects.baseenv["letters"]
@@ -111,7 +103,7 @@ class VectorTestCase(unittest.TestCase):
     def testSetItem(self):
         vec = robjects.r.seq(1, 10)
         vec[0] = 20
-        self.assertEquals(20, vec[0])
+        self.assertEqual(20, vec[0])
 
     def testSetItemOutOfBounds(self):
         vec = robjects.r.seq(1, 10)
@@ -130,7 +122,7 @@ class VectorTestCase(unittest.TestCase):
         r_names = robjects.baseenv["c"](*v_names)
         vec = robjects.baseenv["names<-"](vec, r_names)
         for i in xrange(len(vec)):
-            self.assertEquals(v_names[i], vec.names[i])
+            self.assertEqual(v_names[i], vec.names[i])
 
         vec.names[0] = 'x'
 
@@ -139,44 +131,110 @@ class VectorTestCase(unittest.TestCase):
         names = ['x', 'y', 'z']
         vec.names = names
         for i in xrange(len(vec)):
-            self.assertEquals(names[i], vec.names[i])
+            self.assertEqual(names[i], vec.names[i])
 
-    def testNAinteger(self):
+    def testNAInteger(self):
         vec = robjects.IntVector(range(3))
-        vec[0] = robjects.NA_integer
+        vec[0] = robjects.NA_Integer
         self.assertTrue(robjects.baseenv['is.na'](vec)[0])
-    def testNAreal(self):
+    def testNAReal(self):
         vec = robjects.FloatVector((1.0, 2.0, 3.0))
-        vec[0] = robjects.NA_real
+        vec[0] = robjects.NA_Real
         self.assertTrue(robjects.baseenv['is.na'](vec)[0])
-    def testNAbool(self):
+    def testNALogical(self):
         vec = robjects.BoolVector((True, False, True))
-        vec[0] = robjects.NA_bool
+        vec[0] = robjects.NA_Logical
         self.assertTrue(robjects.baseenv['is.na'](vec)[0])
-    def testNAcomplex(self):
+    def testNAComplex(self):
         vec = robjects.ComplexVector((1+1j, 2+2j, 3+3j))
-        vec[0] = robjects.NA_complex
+        vec[0] = robjects.NA_Complex
         self.assertTrue(robjects.baseenv['is.na'](vec)[0])
-    def testNAcharacter(self):
+    def testNACharacter(self):
         vec = robjects.StrVector('abc')
-        vec[0] = robjects.NA_character
+        vec[0] = robjects.NA_Character
         self.assertTrue(robjects.baseenv['is.na'](vec)[0])
 
+    def testRepr(self):
+        vec = robjects.IntVector((1,2,3))
+        s = repr(vec).split('\n')
+        self.assertEqual('[       1,        2,        3]', s[1])
+
     def testIteritems(self):
         vec = robjects.IntVector(range(3))
         vec.names = robjects.StrVector('abc')
         names = [k for k,v in vec.iteritems()]
-        self.assertEquals(['a', 'b', 'c'], names)
+        self.assertEqual(['a', 'b', 'c'], names)
         values = [v for k,v in vec.iteritems()]
-        self.assertEquals([0, 1, 2], values)
+        self.assertEqual([0, 1, 2], values)
 
     def testIteritemsNoNames(self):
         vec = robjects.IntVector(range(3))
         names = [k for k,v in vec.iteritems()]
-        self.assertEquals([None, None, None], names)
+        self.assertEqual([None, None, None], names)
         values = [v for k,v in vec.iteritems()]
-        self.assertEquals([0, 1, 2], values)
+        self.assertEqual([0, 1, 2], values)
 
+class FactorVectorTestCase(unittest.TestCase):
+    def testNew(self):
+        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
+        self.assertEqual(6, len(vec))
+
+    def testIsordered(self):
+        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
+        self.assertFalse(vec.isordered)
+
+    def testNlevels(self):
+        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
+        self.assertEqual(3, vec.nlevels)
+
+    def testLevels(self):
+        vec = robjects.FactorVector(robjects.StrVector('abaabc'))
+        self.assertEqual(3, len(vec.levels))
+        self.assertEqual(set(('a','b','c')), set(tuple(vec.levels)))
+    
+    def testIter_labels(self):
+        values = 'abaabc'
+        vec = robjects.FactorVector(robjects.StrVector(values))
+        it = vec.iter_labels()
+        for a, b in zip(values, it):
+            self.assertEqual(a, b)
+
+_dateval_tuple = (1984, 1, 6, 6, 22, 0, 1, 6, 0) 
+
+class DateTimeVectorTestCase(unittest.TestCase):
+    
+    def setUp(self):
+        if IS_PYTHON3:
+            self._accept2dyear = time.accept2dyear
+        time.accept2dyear = False
+
+    def tearDown(self):
+        if IS_PYTHON3:
+            time.accept2dyear = self._accept2dyear
+
+    def testPOSIXlt_fromInvalidPythonTime(self):
+        x = [time.struct_time(_dateval_tuple), 
+             time.struct_time(_dateval_tuple)]
+        x.append('foo')
+        self.assertRaises(ValueError, robjects.POSIXlt, x)
+        
+    def testPOSIXlt_fromPythonTime(self):
+        x = [time.struct_time(_dateval_tuple), 
+             time.struct_time(_dateval_tuple)]
+        res = robjects.POSIXlt(x)
+        self.assertEqual(2, len(x))
+
+    def testPOSIXct_fromInvalidPythonTime(self):
+        x = [time.struct_time(_dateval_tuple), 
+             time.struct_time(_dateval_tuple)]
+        x.append('foo')
+        self.assertRaises(ValueError, robjects.POSIXct, x)
+
+    def testPOSIXct_fromPythonTime(self):
+        x = [time.struct_time(_dateval_tuple), 
+             time.struct_time(_dateval_tuple)]
+        res = robjects.POSIXct(x)
+        self.assertEqual(2, len(x))
 
 class ExtractDelegatorTestCase(unittest.TestCase):
 
@@ -194,7 +252,7 @@ class ExtractDelegatorTestCase(unittest.TestCase):
 
         mySubset = mySeq.rx(myIndex)
         for i, si in enumerate(myIndex):
-            self.assertEquals(mySeq[si-1], mySubset[i])
+            self.assertEqual(mySeq[si-1], mySubset[i])
         
     def testExtractByName(self):
         seq_R = robjects.baseenv["seq"]
@@ -210,7 +268,7 @@ class ExtractDelegatorTestCase(unittest.TestCase):
         mySubset = mySeq.rx(myIndex)
 
         for i, si in enumerate(myIndex):
-            self.assertEquals(2, mySubset[i])
+            self.assertEqual(2, mySubset[i])
 
     def testExtractIndexError(self):
         seq_R = robjects.baseenv["seq"]
@@ -227,28 +285,55 @@ class ExtractDelegatorTestCase(unittest.TestCase):
 
        
     def testReplace(self):
-        vec = robjects.r.seq(1, 10)
+        vec = robjects.IntVector(range(1, 6))
         i = array.array('i', [1, 3])
         vec.rx[rlc.TaggedList((i, ))] = 20
-        self.assertEquals(20, vec[0])
-        self.assertEquals(2, vec[1])
-        self.assertEquals(20, vec[2])
-        self.assertEquals(4, vec[3])
+        self.assertEqual(20, vec[0])
+        self.assertEqual(2, vec[1])
+        self.assertEqual(20, vec[2])
+        self.assertEqual(4, vec[3])
 
+        vec = robjects.IntVector(range(1, 6))
         i = array.array('i', [1, 5])
         vec.rx[rlc.TaggedList((i, ))] = 50
-        self.assertEquals(50, vec[0])
-        self.assertEquals(2, vec[1])
-        self.assertEquals(20, vec[2])
-        self.assertEquals(4, vec[3])
-        self.assertEquals(50, vec[4])
-                         
+        self.assertEqual(50, vec[0])
+        self.assertEqual(2, vec[1])
+        self.assertEqual(3, vec[2])
+        self.assertEqual(4, vec[3])
+        self.assertEqual(50, vec[4])
+
+        vec = robjects.IntVector(range(1, 6))
+        vec.rx[1] = 70
+        self.assertEqual(70, vec[0])
+        self.assertEqual(2, vec[1])
+        self.assertEqual(3, vec[2])
+        self.assertEqual(4, vec[3])
+        self.assertEqual(5, vec[4])
+
+        vec = robjects.IntVector(range(1, 6))
+        vec.rx[robjects.IntVector((1, 3))] = 70
+        self.assertEqual(70, vec[0])
+        self.assertEqual(2, vec[1])
+        self.assertEqual(70, vec[2])
+        self.assertEqual(4, vec[3])
+        self.assertEqual(5, vec[4])
+
+
+        m = robjects.r('matrix(1:10, ncol=2)')
+        m.rx[1, 1] = 9
+        self.assertEqual(9, m[0])
+
+        m = robjects.r('matrix(1:10, ncol=2)')
+        m.rx[2, robjects.IntVector((1,2))] = 9
+        self.assertEqual(9, m[1])
+        self.assertEqual(9, m[6])
+                                 
     def testExtractRecyclingRule(self):
         # recycling rule
         v = robjects.Vector(array.array('i', range(1, 23)))
         m = robjects.r.matrix(v, ncol = 2)
         col = m.rx(True, 1)
-        self.assertEquals(11, len(col))
+        self.assertEqual(11, len(col))
 
     def testExtractList(self):
         # list
@@ -261,7 +346,10 @@ class ExtractDelegatorTestCase(unittest.TestCase):
 
 def suite():
     suite = unittest.TestLoader().loadTestsFromTestCase(VectorTestCase)
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(FactorVectorTestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(DateTimeVectorTestCase))
     suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ExtractDelegatorTestCase))
+
     return suite
 
 if __name__ == '__main__':
